/* Demonstate showing messages on oled i2c display via xap-osd schema messages
*/
#include "ets_sys.h"
#include "osapi.h"
#include "gpio.h"
#include "os_type.h"
#include "user_config.h"
#include "user_interface.h"
#include "espconn.h"
#include "mem.h"
#include "driver/uart.h"
#include "mqtt.h"
#include "xap.h"

static volatile os_timer_t client_timer; // also xAP heartbeat
static volatile os_timer_t info_timer; // xAP info timer
static volatile bool OLED;

MQTT_Client mqttClient;

//uint8_t state=0;
//#define LED_GPIO 2
//#define LED_GPIO_MUX PERIPHS_IO_MUX_GPIO2_U
//#define LED_GPIO_FUNC FUNC_GPIO2

/// Decode allowable values for a "state=value" key pair.
//int bscDecodeState(char *statestr) {
//    int i;
//    static char *value[] = {"on","off","true","false","yes","no","1","0","toggle"};
//    static int stateL[] = {1,0,1,0,1,0,1,0,2};
//    if (statestr == NULL) return -1;
//   for(i=0; i < sizeof(value)/sizeof(value[0]); i++) {
//            if(strcasecmp(statestr, value[i]) == 0)
//                    return stateL[i];
//    }
//    return -1;
//}

static void ICACHE_FLASH_ATTR heartbeat(void) {
    #ifdef PLATFORM_DEBUG
    ets_uart_printf("XAP heartbeart\r\n");
    #endif            	

    char buff[XAP_DATA_LEN];
    os_sprintf(buff, "xap-hbeat\n"
                "{\n"
                "v=12\n"
                "hop=1\n"
                "uid=%s\n"
                "class=xap-hbeat.alive\n"
                "source=%s\n"
                "interval=60\n"
                "}\n", XAP_UID, XAP_SOURCE);
	MQTT_Publish(&mqttClient, TOPIC_PUB, buff, strlen(buff), 0, 0);
}        

//static void ICACHE_FLASH_ATTR infoEvent(char *class) {
//    char buff[XAP_DATA_LEN];
//    os_sprintf(buff, "xap-header\n"
//                "{\n"
//                "v=12\n"
//                "hop=1\n"
//                "uid=%s\n"
//                "class=%s\n"
//                "source=%s\n"
//                "}\n"
//				"output.state\n"
//				"{\n"
//				"state=%s\n"
//				"}\n", XAP_UID, class, XAP_SOURCE, state ? "on" : "off");
//	MQTT_Publish(&mqttClient, TOPIC_PUB, buff, strlen(buff), 0, 0);
//}

void mqttConnectedCb(uint32_t *args)
{
	MQTT_Client* client = (MQTT_Client*)args;
	OLED_Print(2, 5, "MQTT: Connected", 1);
	ets_uart_printf("MQTT: Connected\r\n");
	MQTT_Subscribe(client, TOPIC_SOURCE, 0);
}

void mqttDisconnectedCb(uint32_t *args)
{
	MQTT_Client* client = (MQTT_Client*)args;
	OLED_Print(2, 5, "MQTT: Disconnected", 1);
	//ets_uart_printf("MQTT: Disconnected\r\n");
}

void mqttPublishedCb(uint32_t *args)
{
	MQTT_Client* client = (MQTT_Client*)args;
	//OLED_Print(2, 5, "MQTT: Published", 1);
	ets_uart_printf("MQTT: Published\r\n");
}

void mqttDataCb(uint32_t *args, const char* topic, uint32_t topic_len, const char *data, uint32_t data_len)
{
	MQTT_Client* client = (MQTT_Client*)args;

	xAPFrame frame;
	// *data is altered by this call.
	parseMsg(&frame, (unsigned char *)data, data_len);
	
	char *class = xapGetValue(&frame,"xap-header","class");
	if(class == NULL) return;	
	if(strcasecmp(class,"xAP-OSD.Display") == 0) {	
		char *line1Text = xapGetValue(&frame,"Display.Text","Line1");
		char *line2Text = xapGetValue(&frame,"Display.Text","Line2");
		char *line3Text = xapGetValue(&frame,"Display.Text","Line3");
		char *line4Text = xapGetValue(&frame,"Display.Text","Line4");
		char *line5Text = xapGetValue(&frame,"Display.Text","Line5");
		char *line6Text = xapGetValue(&frame,"Display.Text","Line6");
		char *line7Text = xapGetValue(&frame,"Display.Text","Line7");
		char *line8Text = xapGetValue(&frame,"Display.Text","Line8");
		char *line1Size = xapGetValue(&frame,"Display.Text","Size1");
		char *line2Size = xapGetValue(&frame,"Display.Text","Size2");
		char *line3Size = xapGetValue(&frame,"Display.Text","Size3");
		char *line4Size = xapGetValue(&frame,"Display.Text","Size4");
		char *line5Size = xapGetValue(&frame,"Display.Text","Size5");
		char *line6Size = xapGetValue(&frame,"Display.Text","Size6");
		char *line7Size = xapGetValue(&frame,"Display.Text","Size7");
		char *line8Size = xapGetValue(&frame,"Display.Text","Size8");			

		if (line1Text != NULL) {
		if (line1Size == NULL) line1Size = "1";
		if(strcasecmp(line1Size,"2") == 0) { 
			OLED_Print(0, 0, line1Text, 2);
		} else {
			OLED_Print(0, 0, line1Text, 1);
		}
		}

		if (line2Text != NULL) {
		if (line2Size == NULL) line2Size = "1";
		if(strcasecmp(line2Size,"2") == 0) { 
			OLED_Print(0, 1, line2Text, 2);
		} else {
			OLED_Print(0, 1, line2Text, 1);
		}
		}
		
		if (line3Text != NULL) {
		if (line3Size == NULL) line3Size = "1";
		if(strcasecmp(line3Size,"2") == 0) { 
			OLED_Print(0, 2, line3Text, 2);
		} else {
			OLED_Print(0, 2, line3Text, 1);
		}
		}

		if (line4Text != NULL) {
		if (line4Size == NULL) line4Size = "1";
		if(strcasecmp(line4Size,"2") == 0) { 
			OLED_Print(0, 3, line4Text, 2);
		} else {
			OLED_Print(0, 3, line4Text, 1);
		}
		}	

		if (line5Text != NULL) {
		if (line5Size == NULL) line5Size = "1";
		if(strcasecmp(line5Size,"2") == 0) { 
			OLED_Print(0, 4, line5Text, 2);
		} else {
			OLED_Print(0, 4, line5Text, 1);
		}
		}

		if (line6Text != NULL) {
		if (line6Size == NULL) line6Size = "1";
		if(strcasecmp(line6Size,"2") == 0) { 
			OLED_Print(0, 5, line6Text, 2);
		} else {
			OLED_Print(0, 5, line6Text, 1);
		}
		}

		if (line7Text != NULL) {
		if (line7Size == NULL) line7Size = "1";
		if(strcasecmp(line7Size,"2") == 0) { 
			OLED_Print(0, 6, line7Text, 2);
		} else {
			OLED_Print(0, 6, line7Text, 1);
		}
		}
		if (line8Text != NULL) {
		if (line8Size == NULL) line8Size = "1";
		if(strcasecmp(line8Size,"2") == 0) { 
			OLED_Print(0, 7, line8Text, 2);
		} else {
			OLED_Print(0, 7, line8Text, 1);
		}
		}		
	}
}

void xap_init() {
	MQTT_InitConnection(&mqttClient, MQTT_HOST, MQTT_PORT, 0);
	MQTT_InitClient(&mqttClient, XAP_SOURCE, "", "", 120, 1);

	//MQTT_InitLWT(&mqttClient, "/lwt", "offline", 0, 0);
	MQTT_OnConnected(&mqttClient, mqttConnectedCb);
	MQTT_OnDisconnected(&mqttClient, mqttDisconnectedCb);
	MQTT_OnPublished(&mqttClient, mqttPublishedCb);
	MQTT_OnData(&mqttClient, mqttDataCb);

    MQTT_Connect(&mqttClient);
	heartbeat();
	OLED_CLS();
	
	os_timer_disarm(&client_timer);
	os_timer_setfn(&client_timer, (os_timer_func_t *)heartbeat, NULL);
	os_timer_arm(&client_timer, XAP_HEARTBEAT_INTERVAL * 1000, 1);

}

static void ICACHE_FLASH_ATTR wait_for_ip(uint8 flag)
{
	LOCAL struct ip_info ipconfig;
    os_timer_disarm(&client_timer);

    switch(wifi_station_get_connect_status())
	{
    	case STATION_GOT_IP:
            wifi_get_ip_info(STATION_IF, &ipconfig);
            if( ipconfig.ip.addr != 0) {
    			#ifdef PLATFORM_DEBUG
            	ets_uart_printf("WiFi connected\r\n");
    			#endif     
				OLED_Print(2, 0, "Connected!", 2); 
				OLED_Print(2, 2, "IP Obtained", 2);  
				xap_init();
            } else {
                os_timer_setfn(&client_timer, (os_timer_func_t *)wait_for_ip, NULL);
                os_timer_arm(&client_timer, 500, 0);
            }
			break;
		case STATION_WRONG_PASSWORD:
			#ifdef PLATFORM_DEBUG
			ets_uart_printf("WiFi connecting error, wrong password\r\n");
			#endif
            os_timer_setfn(&client_timer, (os_timer_func_t *)wait_for_ip, NULL);
            os_timer_arm(&client_timer, 1000, 0);
			break;
		case STATION_NO_AP_FOUND:
			#ifdef PLATFORM_DEBUG
			ets_uart_printf("WiFi connecting error, ap not found\r\n");
			#endif
            os_timer_setfn(&client_timer, (os_timer_func_t *)wait_for_ip, NULL);
            os_timer_arm(&client_timer, 500, 0);
			break;
		case STATION_CONNECT_FAIL:
			#ifdef PLATFORM_DEBUG
			ets_uart_printf("WiFi connecting fail\r\n");
			#endif
            os_timer_setfn(&client_timer, (os_timer_func_t *)wait_for_ip, NULL);
            os_timer_arm(&client_timer, 1000, 0);
			break;
		default:
			#ifdef PLATFORM_DEBUG
			ets_uart_printf("WiFi connecting...\r\n");
			#endif
            os_timer_setfn(&client_timer, (os_timer_func_t *)wait_for_ip, NULL);
            os_timer_arm(&client_timer, 1000, 0);
	}
}

static void ICACHE_FLASH_ATTR system_is_done(void){
	//Bringing up WLAN
	wifi_station_connect();
	wifi_station_dhcpc_start();
	if(wifi_station_get_auto_connect() == 0)
		wifi_station_set_auto_connect(1);
	//Wait for connection
	os_timer_disarm(&client_timer);
	os_timer_setfn(&client_timer, (os_timer_func_t *)wait_for_ip, NULL);
	os_timer_arm(&client_timer, 100, 0);
}

void ICACHE_FLASH_ATTR setup_wifi_st_mode(void)
{
	wifi_set_opmode((wifi_get_opmode()|STATION_MODE)&STATIONAP_MODE);
	struct station_config stconfig;
	wifi_station_disconnect();
	wifi_station_dhcpc_stop();
	if(wifi_station_get_config(&stconfig))
	{
		os_memset(stconfig.ssid, 0, sizeof(stconfig.ssid));
		os_memset(stconfig.password, 0, sizeof(stconfig.password));
		os_sprintf(stconfig.ssid, "%s", WIFI_SSID);
		os_sprintf(stconfig.password, "%s", WIFI_PASSWORD);
		if(!wifi_station_set_config(&stconfig))
		{
			#ifdef PLATFORM_DEBUG
			ets_uart_printf("ESP8266 not set station config!\r\n");
			#endif
		}
	}
	if(wifi_get_phy_mode() != PHY_MODE_11N)
		wifi_set_phy_mode(PHY_MODE_11N);
	#ifdef PLATFORM_DEBUG
	ets_uart_printf("ESP8266 in STA mode configured.\r\n");
	#endif
}

//Init function
void ICACHE_FLASH_ATTR user_init()
{	
	UARTInit();  // 115200,n,8,1
	os_delay_us(1000);

	i2c_init();
  	OLED = OLED_Init();
	
	setup_wifi_st_mode();
	OLED_Print(2, 0, "Connecting", 2);
	
    //Wait for system to be done.
    system_init_done_cb(&system_is_done);
    //system_os_task()
}
